"""Calculates precision for defined columns in input csv

Required libraries:
-math
-matplotlib.pyplot
-pandas
-scipy.stats

Functions:

"""

import math
import matplotlib.pyplot as plt
import pandas as pd
import scipy.stats as stats

# Input
input_csv = "HCW-ABMv1_0_0-0051 ReplicationAssessment_ScenarioA_default-table_csv_out.csv"
columns = ["[run number]",
           "crossing-success-rate-founder-population"]
title = "Replication assessment: CSR with default levels of factors"
output_csv = "ReplicationAssessment_ScenarioA_default.csv"
confidence = 0.95

# Procedures
def calculate_stepwise_mean(df):
    """Calculates the stepwise mean of a df

    Parameters:
    ------------
    df : DataFrame

    Returns:
    --------
    means_df : DataFrame
    
    """

    i_max = len(df) + 1 # because index begins with 0
    means_df = pd.DataFrame()
    means_df = df[0:1]
    i = 2 # skip the first row for calculation
    while i < i_max: 
        part_df = df[0:i]
        mean = part_df.mean()
        means_df = means_df.append(mean, ignore_index=True)
        i = i + 1
    means_df.columns = ['MEAN']
    print ("DataFrame of Means: ")
    print (str(means_df))
    return means_df

def calculate_stepwise_std(df):
    """Calculates the stepwise std of a df

    Parameters:
    ------------
    df : DataFrame

    Returns:
    --------
    means_df : DataFrame
    
    """

    i_max = len(df) + 1 # because index begins with 0
    std_df = pd.DataFrame()
    std_df = df[0:1]
    i = 2 # skip the first row for calculation
    while i < i_max: 
        part_df = df[0:i]
        std = part_df.std()
        std_df = std_df.append(std, ignore_index=True)
        i = i + 1
    std_df.columns = ['STD']
    print ("DataFrame of STD: ")
    print (str(std_df))
    return std_df

def calculate_stepwise_confidence_interval(df, confidence):
    """Calculates stepwise confidence interval

    The method resembles the KONFIDENZ.NORM function in Excel.

    Parameters:
    -----------
    df : DataFrame
    confidence : float

    Returns:
    --------
    ci_df : DataFrame

    """

    q = (1 - confidence) / 2 + confidence # because the error has two tails
                                          # therefore confidence 0.95 equals
                                          # q = 0.975
    i_max = len(df) + 1 # because index begins with 0
    ci_df = pd.DataFrame()
    i = 1
    while i < i_max:
        n = i
        part_df = df[0:i]
        m = part_df.mean().iloc[0]
        std_err = part_df.std().iloc[0]

        z_critical = stats.norm.ppf(q = q)
        print ("z-critical value:")
        print (str(z_critical))
        margin_of_error = z_critical * (std_err/math.sqrt(n))
        print ("Margin of Error:")
        print (str(margin_of_error))

        ci = (m - margin_of_error, m + margin_of_error)  

        print("N = " + str(n))
        print("MEAN = " + str(m))
        print("STD = " + str(std_err))
        print("CI = " + str(ci) + "\n\n")
        ci_df = ci_df.append([float(margin_of_error)], ignore_index=True)
        i = i + 1
    ci_df.columns = ['CI'] # without this column title would be 0
    print ("ci_df: " + "\n\n" + str(ci_df))
    return ci_df

def calculate_precision_stepwise(means_df, ci_df):
    """Calculates stepwise precision

    Parameters:
    -----------
    means_df : DataFrame 
    ci_df : DataFrame
        confidence intervals

    Returns:
    --------
    prec_df : DataFrame
        precision values between 0 and 1=100%

    """
    
    prec_df = pd.DataFrame()
    i = 0
    i_max = len(df)
    while i < i_max:
        mean = float(means_df.iloc[i])
        ci = float(ci_df.iloc[i])
        if mean != 0:
            prec = float((ci/2)/mean)
        else:
            prec = 0
        print ("Mean = " + str(mean))
        print ("CI = " + str(ci))
        print ("Precision = " + str(prec))
        prec_df = prec_df.append([prec], ignore_index=True)
        i = i + 1
    prec_df.columns = ['PRECISION'] # without this column title would be 0
    print ("prec_df: " + "\n\n" + str(prec_df))
    return prec_df
 
# Execution
## 1. Read input data
df = pd.read_csv(input_csv, index_col=0, usecols=columns)
df = df.sort_index(ascending=True)
print ("Input DataFrame: " + "\n\n" + str(df))

## 2. Calculate statistical values
means_df = calculate_stepwise_mean(df)
std_df = calculate_stepwise_std(df)
ci_df = calculate_stepwise_confidence_interval(df, confidence)
prec_df = calculate_precision_stepwise(means_df, ci_df)

## 3. Create results
results_df = pd.DataFrame()
results_df["MEAN"] = means_df["MEAN"]
results_df["CI"] = ci_df["CI"]
results_df["PRECISION"] = prec_df["PRECISION"]
results_df["ACCEPT_PREC"] = 0.05
results_df.index.name = "RUN"
results_df.to_csv(output_csv)
print ("Results DataFrame: " + "\n\n" + str(results_df))

## 4. Graphical output
#results_df.plot(y="PRECISION", kind='line')
plt.figure(figsize=[9,5])
plt.plot("PRECISION", data=results_df)
plt.plot("ACCEPT_PREC", data=results_df, color='red')
plt.ylim(0,1)
plt.xlim(0,)
plt.axvline(x=27, ymin=0, ymax=1, linestyle = ':', color='red',
            label='selected number of runs') # vertical line
plt.xlabel("number of runs")
plt.ylabel('precision')
plt.legend()
plt.title(title)
plt.savefig("ReplicationAssessment.png")
plt.show()






