"""Calculates statistics for multiple scenarios and multiple parameters

Required libraries:
-matplotlib
-numpy
-os
-pandas
-sys

"""

import matplotlib.pyplot as plt
import numpy as np
import os
import pandas as pd
import sys
from os import listdir

# Inputs
# Files must have format SensitivityScenario_parameter.csv
input_folder = os.getcwd() + '\\input\\'
output_folder = os.getcwd() + '\\output\\'
praefix = 'OneFactorSensitivity_'
scenarios = ['ScenarioA', 'ScenarioB', 'ScenarioC', 'ScenarioD']
calc_values = ["sum-of-target-area-hominins",
               "sum-of-founder-population-size-reached",
               "crossing-success-rate-individuals",
               "crossing-success-rate-founder-population",
               "death-dehydration",
               "death-exhaustion",
               "death-hypothermia",
               "death-left_map_land",
               "death-left_map_water",
               "death-random",
               "sum-of-deaths",
               "attempts-source-water",
               "mean-individual-hours-water-crossing"]
parameters_dict = {
    # paramter : [values list, unit, default value position -> different color in bar chart]
    'basal-metabolic-rate' : [[1000, 1500, 2000, 2500, 3000],'basal-metabolic-rate [energy-units/day]', 1],
    'current-speed' : [[0, 1, 2, 3, 4], 'current-speed [km/h]', 0],
    'death-rate-in-water' : [[0, 0.1, 0.2, 0.3, 0.4], 'death-rate-in-water', 0],
    'decision-randomness' : [[0, 0.1, 0.2, 0.3, 0.4], 'decision-randomness', 1],
    'dehydration' : [[True, False], 'dehydration', 0],
    'hours-until-dehydration' : [[48, 72, 96, 120, 144], 'hours-until-dehydration [h]', 2],
    'max-energy' : [[35000, 40000, 45000, 50000, 55000],'max-energy [energy-units]', 2],
    'number-founder-agents' : [[30, 40, 50, 60, 70], 'number-founder-agents', 2],
    'optimal-walking-speed' : [[3, 4, 5, 6, 7], 'optimal-walking-speed [km/h]', 2],
    'perception-radius' : [[10, 15, 20, 25, 30], 'perception-radius [km]', 2],
    'population-density' : [[0.025, 0.05, 0.1, 0.15, 0.2], 'population-density [individuals/km^2]', 2],
    'water-barrier-size' : [[5, 10, 15, 20, 25], 'water-barrier-size [km]', 1],
    'water-temperature-interval' : [["0-7°C", "8-15°C", "16-23°C", "24-31°C", "32-39°C"], 'water-temperature-interval [°C]', 3]
    }

# For each scenario and parameter
# Parameter value | mean | std  |
# value 1         |      |      |
# value 2         |      |      |
# value 3         |      |      |
# value 4         |      |      |
# value 5         |      |      |
#
# Results table outline option 1
# Parameter value | mean 1 | mean 2 | mean 3 | mean 4 | mean 5 | std 1 | std 2 | std 3 | std 4 | std 5 |
# RanSwi          |        |        |        |        |        |       |       |       |       |       |
# RandDri         |        |        |        |        |        |       |       |       |       |       |
# DirSwi          |        |        |        |        |        |       |       |       |       |       |   
# DirRaf          |        |        |        |        |        |       |       |       |       |       |
#
# Results table outline option 2
# Scenario | RanSwi | RanDri | DirSwi | DirRaf |
# Mean 1   |        |        |        |        |
# Mean 2   |        |        |        |        |
# Mean 3   |        |        |        |        |
# Mean 4   |        |        |        |        |
# Mean 5   |        |        |        |        |
#
# Output csv file: Sensitivity_parameter_calc-value_stats.csv

# Procedure
def define_workspace(input_folder, output_folder):
    has_input_folder = False # intial value
    
    if os.path.isdir(output_folder) == False:
        os.mkdir(output_folder)

    if os.listdir(output_folder) != []:
        print ("Output folder must be empty!")
        sys.exit()
    
    if os.path.isdir(input_folder) == True:
        has_input_folder = True
        
    else:
        has_input_folder = False
        print ("No input folder!")
    
    if has_input_folder == True:
        print ("Input folder: " + input_folder)
        
    print ("Output folder: " + output_folder)

# Execution
define_workspace(input_folder, output_folder)
for calc_value in calc_values:
    print('Calculating statistics for ' + calc_value + ' ...')
    for parameter in parameters_dict:
        print('Parameter: ' + parameter)
        scenario0_file = input_folder + praefix + str(scenarios[0]) + '_' + parameter
        scenario1_file = input_folder + praefix + str(scenarios[1]) + '_' + parameter
        scenario2_file = input_folder + praefix + str(scenarios[2]) + '_' + parameter
        scenario3_file = input_folder + praefix + str(scenarios[3]) + '_' + parameter
        df0 = pd.read_csv(scenario0_file + '.csv')
        df1 = pd.read_csv(scenario1_file + '.csv')
        df2 = pd.read_csv(scenario2_file + '.csv')
        df3 = pd.read_csv(scenario3_file + '.csv')

        print(str(df0))
        print(str(df1))
        print(str(df2))
        print(str(df3))

        parameter_values_list = parameters_dict.get(parameter)[0]
        print('parameter_values_list: ' + str(parameter_values_list))
        
        # Scenario Dataframes:
        # 1. Select column entry by specific parameter value
        # 2. Make sub-dataframe of specific parameter value
        # 2. Calculate statistic of sub-dataframe
        #                  
        # RanSwi
        con00 = df0[parameter] == parameter_values_list[0]
        df00 = df0[con00]
        RanSwi_mean1 = df00[calc_value].mean()
        RanSwi_std1 = df00[calc_value].std()
        con01 = df0[parameter] == parameter_values_list[1]
        df01 = df0[con01]
        RanSwi_mean2 = df01[calc_value].mean()
        RanSwi_std2 = df01[calc_value].std()
        if len(parameter_values_list) == 5:
            con02 = df0[parameter] == parameter_values_list[2]
            df02 = df0[con02]
            RanSwi_mean3 = df02[calc_value].mean()
            RanSwi_std3 = df02[calc_value].std()
            con03 = df0[parameter] == parameter_values_list[3]
            df03 = df0[con03]
            RanSwi_mean4 = df03[calc_value].mean()
            RanSwi_std4 = df03[calc_value].std()
            con04 = df0[parameter] == parameter_values_list[4]
            df04 = df0[con04]
            RanSwi_mean5 = df04[calc_value].mean()
            RanSwi_std5 = df04[calc_value].std()

        # RanDri
        con10 = df1[parameter] == parameter_values_list[0]
        df10 = df1[con10]        
        RanDri_mean1 = df10[calc_value].mean()
        RanDri_std1 = df10[calc_value].std()
        con11 = df1[parameter] == parameter_values_list[1]       
        df11 = df1[con11]
        RanDri_mean2 = df11[calc_value].mean()
        RanDri_std2 = df11[calc_value].std()
        if len(parameter_values_list) == 5:
            con12 = df1[parameter] == parameter_values_list[2]
            df12 = df1[con12]
            RanDri_mean3 = df12[calc_value].mean()
            RanDri_std3 = df12[calc_value].std()
            con13 = df1[parameter] == parameter_values_list[3]
            df13 = df1[con13]
            RanDri_mean4 = df13[calc_value].mean()
            RanDri_std4 = df13[calc_value].std()
            con14 = df1[parameter] == parameter_values_list[4]
            df14 = df1[con14]
            RanDri_mean5 = df14[calc_value].mean()
            RanDri_std5 = df14[calc_value].std()

        # DirSwi
        con20 = df2[parameter] == parameter_values_list[0]
        df20 = df2[con20]        
        DirSwi_mean1 = df20[calc_value].mean()
        DirSwi_std1 = df20[calc_value].std()
        con21 = df2[parameter] == parameter_values_list[1]
        df21 = df2[con21]        
        DirSwi_mean2 = df21[calc_value].mean()
        DirSwi_std2 = df21[calc_value].std()
        if len(parameter_values_list) == 5:
            con22 = df2[parameter] == parameter_values_list[2]
            df22 = df2[con22]        
            DirSwi_mean3 = df22[calc_value].mean()
            DirSwi_std3 = df22[calc_value].std()
            con23 = df2[parameter] == parameter_values_list[3]
            df23 = df2[con23]        
            DirSwi_mean4 = df23[calc_value].mean()
            DirSwi_std4 = df23[calc_value].std()
            con24 = df2[parameter] == parameter_values_list[4]
            df24 = df2[con24]        
            DirSwi_mean5 = df24[calc_value].mean()
            DirSwi_std5 = df24[calc_value].std()

        # DirRaf
        con30 = df3[parameter] == parameter_values_list[0]
        df30 = df3[con30]
        DirRaf_mean1 = df30[calc_value].mean()
        DirRaf_std1 = df30[calc_value].std() 
        con31 = df3[parameter] == parameter_values_list[1]
        df31 = df3[con31]
        DirRaf_mean2 = df31[calc_value].mean()
        DirRaf_std2 = df31[calc_value].std() 
        if len(parameter_values_list) == 5:
            con32 = df3[parameter] == parameter_values_list[2]
            df32 = df3[con32]
            DirRaf_mean3 = df32[calc_value].mean()
            DirRaf_std3 = df32[calc_value].std() 
            con33 = df3[parameter] == parameter_values_list[3]
            df33 = df3[con33]
            DirRaf_mean4 = df33[calc_value].mean()
            DirRaf_std4 = df33[calc_value].std() 
            con34 = df3[parameter] == parameter_values_list[4]
            df34 = df3[con34]
            DirRaf_mean5 = df34[calc_value].mean()
            DirRaf_std5 = df34[calc_value].std() 

        if len(parameter_values_list) == 2:
            results_means_dict = {
                scenarios[0] : [RanSwi_mean1, RanSwi_mean2],
                scenarios[1] : [RanDri_mean1, RanDri_mean2],
                scenarios[2] : [DirSwi_mean1, DirSwi_mean2],
                scenarios[3] : [DirRaf_mean1, DirRaf_mean2],
                }
            results_stds_dict = {
                scenarios[0] : [RanSwi_std1, RanSwi_std2],
                scenarios[1] : [RanDri_std1, RanDri_std2],
                scenarios[2] : [DirSwi_std1, DirSwi_std2],
                scenarios[3] : [DirRaf_std1, DirRaf_std2],
                }
        
        if len(parameter_values_list) == 5:
            results_means_dict = {
                scenarios[0] : [RanSwi_mean1, RanSwi_mean2, RanSwi_mean3, RanSwi_mean4, RanSwi_mean5],
                scenarios[1] : [RanDri_mean1, RanDri_mean2, RanDri_mean3, RanDri_mean4, RanDri_mean5],
                scenarios[2] : [DirSwi_mean1, DirSwi_mean2, DirSwi_mean3, DirSwi_mean4, DirSwi_mean5],
                scenarios[3] : [DirRaf_mean1, DirRaf_mean2, DirRaf_mean3, DirRaf_mean4, DirRaf_mean5],
                }
            results_stds_dict = {
                scenarios[0] : [RanSwi_std1, RanSwi_std2, RanSwi_std3, RanSwi_std4, RanSwi_std5],
                scenarios[1] : [RanDri_std1, RanDri_std2, RanDri_std3, RanDri_std4, RanDri_std5],
                scenarios[2] : [DirSwi_std1, DirSwi_std2, DirSwi_std3, DirSwi_std4, DirSwi_std5],
                scenarios[3] : [DirRaf_std1, DirRaf_std2, DirRaf_std3, DirRaf_std4, DirRaf_std5],
                }
            
        # Results dataframe
        results_means_df = pd.DataFrame(results_means_dict)
        print('Results of means dataframe: ')
        print(str(results_means_df) + '\n')
        results_means_csv = output_folder + calc_value +  '_' + parameter + '_means' + '.csv'
        results_means_df.to_csv(results_means_csv)
        print(results_means_csv + ' created. \n')

        results_stds_df = pd.DataFrame(results_stds_dict)
        print('Results of stds dataframe: ')
        print(str(results_stds_df) + '\n')
        results_stds_csv = output_folder + calc_value +  '_' + parameter + '_stds' + '.csv'
        results_stds_df.to_csv(results_stds_csv)
        print(results_stds_csv + ' created. \n')

        # Grouped bar chart
        x = np.arange(len(parameter_values_list)) # the label locations
        width = 0.75 # the width of the bars

        fig, ax = plt.subplots(figsize=(10,5))

        y1_means = results_means_df[scenarios[0]].tolist()
        y2_means = results_means_df[scenarios[1]].tolist()
        y3_means = results_means_df[scenarios[2]].tolist()
        y4_means = results_means_df[scenarios[3]].tolist()
        y1_stds = results_stds_df[scenarios[0]].tolist()
        y2_stds = results_stds_df[scenarios[1]].tolist()
        y3_stds = results_stds_df[scenarios[2]].tolist()
        y4_stds = results_stds_df[scenarios[3]].tolist()

        # play with width values to arrange the bars
        rects1 = ax.bar(x - width/2 + width/8, y1_means, width/4, label='A',
                        yerr=y1_stds, color=(0.25, 0.25, 0.25), edgecolor='black')
        rects2 = ax.bar(x - width/4/2, y2_means, width/4, label='B',
                        yerr=y2_stds, color=(0.50, 0.50, 0.50), edgecolor='black')
        rects3 = ax.bar(x + width/4/2, y3_means, width/4, label='C',
                        yerr=y3_stds, color=(0.75, 0.75, 0.75), edgecolor='black')
        rects4 = ax.bar(x + width/2 - width/8, y4_means, width/4, label='D',
                        yerr=y4_stds, color=(1.0, 1.0, 1.0), edgecolor='black')

        # Add some text for labels, title and custom x-axis tick labels, etc.
        xlabel = parameters_dict.get(parameter)[1]
        ax.set_xlabel (xlabel, fontsize=14)
        y_label = calc_value
        if calc_value == 'crossing-success-rate-founder-population':
            y_label = 'CSR'
            ax.set_ylim(0,1.2)
        ax.set_ylabel(y_label, fontsize=14)
        ax.set_xticks(x)
        if parameter_values_list == [True, False]:
            ax.set_xticklabels(["on", "off"])
        else:
            ax.set_xticklabels(parameter_values_list)
        #ax.legend(['A', 'B', 'C', 'D'], fontsize=14)
        ax.tick_params(labelsize=14)

        fig.tight_layout()
        results_png = output_folder + calc_value +  '_' + parameter + '_stats' + '.png'
        plt.savefig(results_png, dpi=300)
        plt.close()
        print(results_png + ' created.' + '\n')




                    
                
        


















