"""Calculates mean and std for each scenario and for each factor

The input files have the structure:

SensitivityScenario_factor.csv

And the output files have the structures:

SensitivityScenario_factor_mean.csv
SensitivityScenario_factor_std.csv

Furthermore, the output files have the table structure:
_______________________________________________________________
__________________|_Level0_|_Level1_|_Level2_|_Level3_|_Level4_|
death-dehydration_|________|________|________|________|________|   
death-removed_____|________|________|________|________|________|
death-exhaustion__|________|________|________|________|________|
death-hypothermia_|________|________|________|________|________|
death-random______|________|________|________|________|________|

"""

import os
import pandas as pd

# Inputs
praefix = 'OneFactorSensitivity_'
scenarios = ['ScenarioA', 'ScenarioB', 'ScenarioC', 'ScenarioD']
suffix = '.csv'
input_folder = os.getcwd() + '\\input\\'
output_folder = os.getcwd() + '\\output\\'
factors_dict = {
    # paramter : [values list, unit, default value position -> different color in bar chart]
    'basal-metabolic-rate' : [[1000, 1500, 2000, 2500, 3000],'basal-metabolic-rate [energy-units/day]', 1],
    'current-speed' : [[0, 1, 2, 3, 4], 'current-speed [km/h]', 0],
    'death-rate-in-water' : [[0, 0.1, 0.2, 0.3, 0.4], 'death-rate-in-water', 0],
    'decision-randomness' : [[0, 0.1, 0.2, 0.3, 0.4], 'decision-randomness', 1],
    'dehydration' : [[True, False], 'dehydration', 0],
    'hours-until-dehydration' : [[48, 72, 96, 120, 144], 'hours-until-dehydration [h]', 2],
    'max-energy' : [[35000, 40000, 45000, 50000, 55000],'max-energy [energy-units]', 2],
    'number-founder-agents' : [[30, 40, 50, 60, 70], 'number-founder-agents', 2],
    'optimal-walking-speed' : [[3, 4, 5, 6, 7], 'optimal-walking-speed [km/h]', 2],
    'perception-radius' : [[10, 15, 20, 25, 30], 'perception-radius [km]', 2],
    'population-density' : [[0.025, 0.05, 0.1, 0.15, 0.2], 'population-density [individuals/km^2]', 2],
    'water-barrier-size' : [[5, 10, 15, 20, 25], 'water-barrier-size [km]', 1],
    'water-temperature-interval' : [["0-7°C", "8-15°C", "16-23°C", "24-31°C", "32-39°C"], 'water-temperature-interval [°C]', 3]
    }

# Procedures
def define_workspace(input_folder, output_folder):
    has_input_folder = False # intial value
    if os.path.isdir(output_folder) == False:
        os.mkdir(output_folder)
    if os.listdir(output_folder) != []:
        print ("Output folder must be empty!")
        sys.exit()
    if os.path.isdir(input_folder) == True:
        has_input_folder = True     
    else:
        has_input_folder = False
        print ("No input folder!")    
    if has_input_folder == True:
        print ("Input folder: " + input_folder)        
    print ("Output folder: " + output_folder)

def CalcStats(input_folder, output_folder, praefix,
              scenario, suffix, factors_dict):
    """Calculates the mean and stds for the death responses of the input
    scenario

    The inputs determine the csv file, the scenarios, the factor,
    and the factor levels. 

    """

    print('Calculating statistics for ' + scenario + ' ...')
    for factor in list(factors_dict.keys()):
        input_csv = input_folder + praefix + scenario + '_' + str(factor) + suffix  
        df = pd.read_csv(input_csv)
        print(input_csv + ' Dataframe: \n' + str(df))

        levels = factors_dict[factor][0]
        print ('Factor : ' + factor)
        print ('Levels : ' + str(levels))

        
        # mean
        # level 0
        con = df[factor] == levels[0]
        df_level0 = df[con]
        mean00 = df_level0['death-dehydration'].mean()
        mean01 = df_level0['death-exhaustion'].mean()
        mean02 = df_level0['death-hypothermia'].mean()
        mean03 = df_level0['death-left_map_land'].mean()
        mean04 = df_level0['death-left_map_water'].mean()
        mean05 = df_level0['death-random'].mean()

        # level 1
        con = df[factor] == levels[1]
        df_level1 = df[con]
        mean10 = df_level1['death-dehydration'].mean()
        mean11 = df_level1['death-exhaustion'].mean()
        mean12 = df_level1['death-hypothermia'].mean()
        mean13 = df_level1['death-left_map_land'].mean()
        mean14 = df_level1['death-left_map_water'].mean()
        mean15 = df_level1['death-random'].mean()
        
        if len(levels) == 5:
            # level 2
            con = df[factor] == levels[2]
            df_level2 = df[con]
            mean20 = df_level2['death-dehydration'].mean()
            mean21 = df_level2['death-exhaustion'].mean()
            mean22 = df_level2['death-hypothermia'].mean()
            mean23 = df_level2['death-left_map_land'].mean()
            mean24 = df_level2['death-left_map_water'].mean()
            mean25 = df_level2['death-random'].mean()

            # level 3
            con = df[factor] == levels[3]
            df_level3 = df[con]
            mean30 = df_level3['death-dehydration'].mean()
            mean31 = df_level3['death-exhaustion'].mean()
            mean32 = df_level3['death-hypothermia'].mean()
            mean33 = df_level3['death-left_map_land'].mean()
            mean34 = df_level3['death-left_map_water'].mean()
            mean35 = df_level3['death-random'].mean()

            # level 4
            con = df[factor] == levels[4]
            df_level4 = df[con]
            mean40 = df_level4['death-dehydration'].mean()
            mean41 = df_level4['death-exhaustion'].mean()
            mean42 = df_level4['death-hypothermia'].mean()
            mean43 = df_level4['death-left_map_land'].mean()
            mean44 = df_level4['death-left_map_water'].mean()
            mean45 = df_level4['death-random'].mean()

        if len(levels) == 2:
            stats_dict = {
                'level'               : [levels[0], levels[1]],
                'death-dehydration'   : [mean00, mean10],
                'death-exhaustion'   : [mean01, mean11],
                'death-hypothermia'   : [mean02, mean12],
                'death-left_map_land' : [mean03, mean13],
                'death-left_map_water': [mean04, mean14],
                'death-random'        : [mean05, mean15]
                }

        if len(levels) == 5:
            stats_dict = {
                'level'               : [levels[0], levels[1], levels[2], levels[3], levels[4]],
                'death-dehydration'   : [mean00, mean10, mean20, mean30, mean40],
                'death-exhaustion'    : [mean01, mean11, mean21, mean31, mean41],
                'death-hypothermia'   : [mean02, mean12, mean22, mean32, mean42],
                'death-left_map_land' : [mean03, mean13, mean23, mean33, mean43],
                'death-left_map_water': [mean04, mean14, mean24, mean34, mean44],
                'death-random'        : [mean05, mean15, mean25, mean35, mean45]
                }
        print('Stats dictionary: \n' + str(stats_dict))
        stats_df = pd.DataFrame(stats_dict)
        print('Stats Dataframe: \n' + str(stats_df))
        output_csv = output_folder + praefix + scenario + '_' + str(factor)+ '_means' + suffix
        stats_df.to_csv(output_csv)
        print (output_csv + ' created.')

        # std
        # level 0
        con = df[factor] == levels[0]
        df_level0 = df[con]
        std00 = df_level0['death-dehydration'].std()
        std01 = df_level0['death-exhaustion'].std()
        std02 = df_level0['death-hypothermia'].std()
        std03 = df_level0['death-left_map_land'].std()
        std04 = df_level0['death-left_map_water'].std()
        std05 = df_level0['death-random'].std()

        # level 1
        con = df[factor] == levels[1]
        df_level1 = df[con]
        std10 = df_level1['death-dehydration'].std()
        std11 = df_level1['death-exhaustion'].std()
        std12 = df_level1['death-hypothermia'].std()
        std13 = df_level1['death-left_map_land'].std()
        std14 = df_level1['death-left_map_water'].std()
        std15 = df_level1['death-random'].std()
        
        if len(levels) == 5:
            # level 2
            con = df[factor] == levels[2]
            df_level2 = df[con]
            std20 = df_level2['death-dehydration'].std()
            std21 = df_level2['death-exhaustion'].std()
            std22 = df_level2['death-hypothermia'].std()
            std23 = df_level2['death-left_map_land'].std()
            std24 = df_level2['death-left_map_water'].std()
            std25 = df_level2['death-random'].std()

            # level 3
            con = df[factor] == levels[3]
            df_level3 = df[con]
            std30 = df_level3['death-dehydration'].std()
            std31 = df_level3['death-exhaustion'].std()
            std32 = df_level3['death-hypothermia'].std()
            std33 = df_level3['death-left_map_land'].std()
            std34 = df_level3['death-left_map_water'].std()
            std35 = df_level3['death-random'].std()

            # level 4
            con = df[factor] == levels[4]
            df_level4 = df[con]
            std40 = df_level4['death-dehydration'].std()
            std41 = df_level4['death-exhaustion'].std()
            std42 = df_level4['death-hypothermia'].std()
            std43 = df_level4['death-left_map_land'].std()
            std44 = df_level4['death-left_map_water'].std()
            std45 = df_level4['death-random'].std()

        if len(levels) == 2:
            stats_dict = {
                'level'               : [levels[0], levels[1]],
                'death-dehydration'   : [std00, std10],
                'death-exhaustion'    : [std01, std11],
                'death-hypothermia'   : [std02, std12],
                'death-left_map_land' : [std03, std13],
                'death-left_map_water': [std04, std14],
                'death-random'        : [std05, std15]
                }

        if len(levels) == 5:
            stats_dict = {
                'level'                : [levels[0], levels[1], levels[2], levels[3], levels[4]],
                'death-dehydration'    : [std00, std10, std20, std30, std40],
                'death-exhaustion'     : [std01, std11, std21, std31, std41],
                'death-hypothermia'    : [std02, std12, std22, std32, std42],
                'death-left_map_land'  : [std03, std13, std23, std33, std43],
                'death-left_map_water' : [std04, std14, std24, std34, std44],
                'death-random'         : [std05, std15, std25, std35, std45]
                }
        print('Stats dictionary: \n' + str(stats_dict))
        stats_df = pd.DataFrame(stats_dict)
        print('Stats Dataframe: \n' + str(stats_df))
        output_csv = output_folder + praefix + scenario + '_' + str(factor)+ '_stds' + suffix
        stats_df.to_csv(output_csv)
        print (output_csv + ' created.')

    print('Finished calculation of stats for ' + scenario + '. \n')
    
    
# Execution
define_workspace(input_folder, output_folder)

for scenario in scenarios:
    CalcStats(input_folder, output_folder, praefix,
              scenario, suffix, factors_dict)
