"""Makes stacked bar chart from input csv

The input csv must have the structure:
_______________________________________________________________
__________________|_Level0_|_Level1_|_Level2_|_Level3_|_Level4_|
death-mapborder___|________|________|________|________|________|   
death-founder_____|________|________|________|________|________|
death-dehydration_|________|________|________|________|________|
death-starvation__|________|________|________|________|________|

"""

import matplotlib.pyplot as plt
import numpy as np
import os
import pandas as pd
import sys
from os import listdir
from operator import add

# Inputs
input_folder = os.getcwd() + '\\input\\'
output_folder = os.getcwd() + '\\output\\'
praefix = 'OneFactorSensitivity_'
scenarios = ['ScenarioA', 'ScenarioB', 'ScenarioC', 'ScenarioD']
suffix = '_means.csv'
factors_dict = {
    # paramter : [values list, unit, default value position -> different color in bar chart]
    'basal-metabolic-rate' : [[1000, 1500, 2000, 2500, 3000],'basal-metabolic-rate [energy-units/day]', 1],
    'current-speed' : [[0, 1, 2, 3, 4], 'current-speed [km/h]', 0],
    'death-rate-in-water' : [[0, 0.1, 0.2, 0.3, 0.4], 'death-rate-in-water', 0],
    'decision-randomness' : [[0, 0.1, 0.2, 0.3, 0.4], 'decision-randomness', 1],
    'dehydration' : [[True, False], 'dehydration', 0],
    'hours-until-dehydration' : [[48, 72, 96, 120, 144], 'hours-until-dehydration [h]', 2],
    'max-energy' : [[35000, 40000, 45000, 50000, 55000],'max-energy [energy-units]', 2],
    'number-founder-agents' : [[30, 40, 50, 60, 70], 'number-founder-agents', 2],
    'optimal-walking-speed' : [[3, 4, 5, 6, 7], 'optimal-walking-speed [km/h]', 2],
    'perception-radius' : [[10, 15, 20, 25, 30], 'perception-radius [km]', 2],
    'population-density' : [[0.025, 0.05, 0.1, 0.15, 0.2], 'population-density [individuals/km^2]', 2],
    'water-barrier-size' : [[5, 10, 15, 20, 25], 'water-barrier-size [km]', 1],
    'water-temperature-interval' : [["0-7°C", "8-15°C", "16-23°C", "24-31°C", "32-39°C"], 'water-temperature-interval [°C]', 3]
    }

# Procedure
def define_workspace(input_folder, output_folder):
    has_input_folder = False # intial value
    if os.path.isdir(output_folder) == False:
        os.mkdir(output_folder)
    if os.listdir(output_folder) != []:
        print ("Output folder must be empty!")
        sys.exit()
    if os.path.isdir(input_folder) == True:
        has_input_folder = True     
    else:
        has_input_folder = False
        print ("No input folder!")    
    if has_input_folder == True:
        print ("Input folder: " + input_folder)        
    print ("Output folder: " + output_folder)

def MakeStackedBarChart(input_folder, output_folder, praefix, scenarios, suffix, factors_dict):
    for scenario in scenarios:
        for factor in list(factors_dict.keys()):
            input_csv = input_folder + praefix + scenario + '_' + factor + suffix
            df = pd.read_csv(input_csv, usecols=['level',
                                                 'death-dehydration',
                                                 'death-exhaustion',
                                                 'death-hypothermia',
                                                 'death-left_map_land',
                                                 'death-left_map_water',
                                                 'death-random'])
            print('Dataframe from input_csv: \n' + str(df))

            ys_dehydration = df['death-dehydration'].tolist()
            ys_exhaustion = df['death-exhaustion'].tolist()
            ys_hypothermia = df['death-hypothermia'].tolist()
            ys_left_map_land = df['death-left_map_land'].tolist()
            ys_left_map_water = df['death-left_map_water'].tolist()
            ys_random = df['death-random'].tolist()

            x_values = df['level'].tolist()
            x_ticks = np.arange(len(x_values))
            
            # Heights of bars
            bottom1 = list(map(add, ys_dehydration, ys_exhaustion))
            bottom2 = list(map(add, bottom1, ys_hypothermia))
            bottom3 = list(map(add, bottom2, ys_left_map_land))
            bottom4 = list(map(add, bottom3, ys_left_map_water))

            fig, ax = plt.subplots()
            y0 = ax.bar(x_ticks, ys_dehydration, color='yellow')
            y1 = ax.bar(x_ticks, ys_exhaustion, bottom=ys_dehydration, color='orange')
            y2 = ax.bar(x_ticks, ys_hypothermia, bottom=bottom1, color='red')
            y3 = ax.bar(x_ticks, ys_left_map_land, bottom=bottom2, color='brown')
            y4 = ax.bar(x_ticks, ys_left_map_water, bottom=bottom3, color='blue')
            y5 = ax.bar(x_ticks, ys_random, bottom=bottom4, color='gray')
                           
            ax.legend((y0[0], y1[0], y2[0], y3[0], y4[0], y5[0]), ('dehydration', 'exhaustion', 'hypothermia', 'left_map_land', 'left_map_water', 'random'),
                 ncol=4, loc='upper center', fontsize='small')
            ax.set_xticks(x_ticks)
            ax.set_xticklabels(x_values)
            ax.set_xlabel(factors_dict[factor][1])
            ax.set_ylabel('# disappearances')
            scenario_titles = {
                'ScenarioA' : 'Scenario A',
                'ScenarioB' : 'Scenario B',
                'ScenarioC' : 'Scenario C',
                'ScenarioD' : 'Scenario D'
                }
            ax.set_title(scenario_titles[scenario])
            ax.set_ylim(0,12000)
            output_png = output_folder + praefix + scenario + '_' + str(factor) + '_means' + '.png'
            plt.savefig(output_png)
            print(output_png + ' created. \n')
            plt.close()

# Execution
define_workspace(input_folder, output_folder)
MakeStackedBarChart(input_folder, output_folder, praefix, scenarios, suffix, factors_dict)


















    
